@@ -20,7 +20,7 @@
  * is" without express or implied warranty.
  *
  * pool_proto_modules.h.: header file for pool_proto_modules.c, pool_proto2.c
- * and pool_process_qeury.c
+ * and pool_process_query.c
  *
  */
 
@@ -96,9 +96,6 @@ extern POOL_STATUS CloseComplete(POOL_CONNECTION *frontend,
 extern POOL_STATUS CommandComplete(POOL_CONNECTION *frontend,
 								   POOL_CONNECTION_POOL *backend);
 
-extern POOL_STATUS ParameterDescription(POOL_CONNECTION *frontend,
-										POOL_CONNECTION_POOL *backend);
-
 extern POOL_STATUS ErrorResponse3(POOL_CONNECTION *frontend,
 								  POOL_CONNECTION_POOL *backend);
 
