@@ -232,7 +232,7 @@ void *pool_search_relcache(POOL_RELCACHE *relcache, POOL_CONNECTION_POOL *backen
 			relcache->cache[index].expire = 0;
 		}
 		/*
-		 * Call user defined unregister/register fuction.
+		 * Call user defined unregister/register function.
 		 */
 		(*relcache->unregister_func)(relcache->cache[index].data);
 		relcache->cache[index].data = result;
@@ -262,12 +262,23 @@ void *int_unregister_func(void *data)
 
 void *string_register_func(POOL_SELECT_RESULT *res)
 {
-	return (res->numrows > 0)? strdup(res->data[0]): NULL;
+	char	*ret = NULL;
+	int		 i;
+
+	if (res->numrows == 0)
+		return NULL;
+
+	for (i = 0; i < res->numrows; i++)
+	{
+		ret = strdup(res->data[i]);
+		break;
+	}
+
+	return (void *)ret;
 }
 
 void *string_unregister_func(void *data)
 {
-    if(data)
-	    free(data);
-    return (void *)0;
+	free(data);
+	return data;
 }
