@@ -30,6 +30,7 @@
 #include <netdb.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <errno.h>
 #include "pool.h"
 #include "pool_config.h"
 #include "watchdog.h"
@@ -43,7 +44,7 @@ static void * exec_ping(void * arg);
 static double get_result (char * ping_data);
 
 /**
- * Try to conect to trusted servers.
+ * Try to connect to trusted servers.
  */
 int
 wd_is_upper_ok(char * server_list)
@@ -61,10 +62,17 @@ wd_is_upper_ok(char * server_list)
 
 	if (server_list == NULL)
 	{
+		pool_error("wd_is_upper_ok: server_list is NULL");
 		return WD_NG;
 	}
 	len = strlen(server_list)+2;
 	buf = malloc(len);
+	if (buf == NULL)
+	{
+		pool_error("wd_is_upper_ok: malloc failed");
+		return WD_NG;
+	}
+
 	memset(buf,0,len);
 	strlcpy(buf,server_list,len);
 	/* thread init */
@@ -101,14 +109,14 @@ wd_is_upper_ok(char * server_list)
 	pthread_attr_destroy(&attr);
 	for (i=0; i <cnt; )
 	{
-		void * result;
-		rc = pthread_join(thread[i], &result);
+		int result;
+		rc = pthread_join(thread[i], (void **)&result);
 		if ((rc != 0) && (errno == EINTR))
 		{
 			usleep(100);
 			continue;
 		}
-		if (result == (void *)WD_OK)
+		if (result == WD_OK)
 		{
 			rtn = WD_OK;
 		}
@@ -119,7 +127,7 @@ wd_is_upper_ok(char * server_list)
 }
 
 /**
- * check if IP addres is unused.
+ * check if IP address is unused.
  */
 int
 wd_is_unused_ip(char * ip)
@@ -130,7 +138,7 @@ wd_is_unused_ip(char * ip)
 	WdInfo thread_arg;
 
 	int rtn = WD_NG;
-	void * result;
+	int result;
 
 	if (ip == NULL)
 	{
@@ -147,12 +155,12 @@ wd_is_unused_ip(char * ip)
 	rc = pthread_create(&thread, &attr, exec_ping, (void*)&thread_arg);
 	pthread_attr_destroy(&attr);
 
-	rc = pthread_join(thread, &result);
+	rc = pthread_join(thread, (void **)&result);
 	if ((rc != 0) && (errno == EINTR))
 	{
 		return WD_NG;
 	}
-	if (result == (void *)WD_NG)
+	if (result == WD_NG)
 	{
 		rtn = WD_OK;
 	}
@@ -161,6 +169,9 @@ wd_is_unused_ip(char * ip)
 }
 
 
+/**
+ * Thread to execute ping against "trusted hosts" or delegate IP.
+ */
 static void *
 exec_ping(void * arg)
 {
@@ -177,10 +188,11 @@ exec_ping(void * arg)
 	snprintf(ping_path,sizeof(ping_path),"%s/ping",pool_config->ping_path);
 	thread_arg = (WdInfo *)arg;
 	memset(result,0,sizeof(result));
+
 	if (pipe(pfd) == -1)
 	{
-		fprintf(stderr,"pipe open error:%s\n",strerror(errno));
-		return NULL;
+		pool_error("exec_ping: pipe open error:%s", strerror(errno));
+		return WD_NG;
 	}
 
 	args[i++] = "ping";
@@ -190,12 +202,23 @@ exec_ping(void * arg)
 	args[i++] = NULL;
 
 	pid = fork();
+	if (pid == -1)
+	{
+		pool_error("exec_ping: fork() failed. reason: %s", strerror(errno));
+		exit(1);
+	}
 	if (pid == 0)
 	{
 		close(STDOUT_FILENO);
 		dup2(pfd[1], STDOUT_FILENO);
 		close(pfd[0]);
 		status = execv(ping_path,args);
+
+		if (status == -1)
+		{
+			pool_error("exec_ping: execv(%s) failed. reason: %s", ping_path, strerror(errno));
+			exit(1);
+		}
 		exit(0);
 	}
 	else
@@ -204,19 +227,35 @@ exec_ping(void * arg)
 		for (;;)
 		{
 			int r;
-			r = waitpid(pid, &status, 0);
+			r = wait(&status);
 			if (r < 0)
 			{
 				if (errno == EINTR)
 					continue;
-				return NULL;
+
+				pool_error("exec_ping: wait() failed. reason: %s", strerror(errno));
+				close(pfd[0]);
+				return WD_NG;
 			}
 
-			if (WIFEXITED(status) == 0 || WEXITSTATUS(status) != 0)
-				return NULL;
+			if (WIFEXITED(status) == 0)
+			{
+				pool_error("exec_ping: %s exited abnormally", ping_path);
+				close(pfd[0]);
+				return WD_NG;
+			}
+			else if (WEXITSTATUS(status) != 0)
+			{
+				pool_debug("exec_ping: failed to ping %s", thread_arg->hostname);
+				return WD_NG;
+			}
 			else
+			{
+				pool_debug("exec_ping: succeed to ping %s", thread_arg->hostname);
 				break;
+			}
 		}
+
 		i = 0;
 		while  (( (r_size = read (pfd[0], &result[i], sizeof(result)-i-1)) > 0) && (errno == EINTR))
 		{
@@ -233,6 +272,9 @@ exec_ping(void * arg)
 	pthread_exit((void *)rtn);
 }
 
+/**
+ * Get average round-trip time of ping result.
+ */
 static double
 get_result (char * ping_data)
 {
@@ -243,11 +285,15 @@ get_result (char * ping_data)
 
 	if (ping_data == NULL)
 	{
+		pool_error("get_result: no ping data");
 		return -1;
 	}
+
+	pool_debug("get_result: ping data: %s", ping_data);
+
 	/*
 	 skip result until average data
-	 tipical result of ping is as follows,
+	 typical result of ping is as follows,
 	 "rtt min/avg/max/mdev = 0.045/0.045/0.046/0.006 ms"
 	 we can find the average data beyond the 4th '/'.
 	 */
@@ -261,18 +307,24 @@ get_result (char * ping_data)
 		}
 		sp ++;
 	}
+
 	ep = strchr (sp,'/');
 	if (ep == NULL)
 	{
 		return -1;
 	}
+
 	*ep = '\0';
 	errno = 0;
+
 	/* convert to numeric data from text */
 	msec = strtod(sp,(char **)NULL);
+
 	if (errno != 0)
 	{
+		pool_error("get_result: strtod() failed. reason: %s", strerror(errno));
 		return -1;
 	}
+
 	return msec;
 }
